@@ -1,9 +1,12 @@
+//! This library was built to help test systems that use libraries which don't provide any
+//! testing utilities themselves. It works by overriding the proxy and root ca attributes
+//! and intercepting proxy requests, then returning mock responses defined by the user
+
 use crate::mock::Response;
 use log::{error, info};
 use native_tls::TlsStream;
 use openssl::pkey::{PKey, PKeyRef, Private};
 use openssl::x509::X509Ref;
-use std::fmt::Write;
 use std::io::{Read, Write as IOWrite};
 use std::net::{SocketAddr, TcpListener, TcpStream};
 use std::sync::mpsc;
@@ -15,6 +18,7 @@ pub use crate::mock::Mock;
 
 const SERVER_ADDRESS_INTERNAL: &str = "127.0.0.1:1234";
 
+/// Primary interface for the library
 pub struct Proxy {
     mocks: Vec<Mock>,
     listening_addr: Option<SocketAddr>,
@@ -43,6 +47,9 @@ impl Proxy {
         Self::default()
     }
 
+    /// Register a given mock with the proxy
+    ///
+    /// Will panic if proxy has already been started
     pub fn register(&mut self, mock: Mock) {
         if self.started {
             panic!("Cannot add mocks to a started proxy");
@@ -50,32 +57,37 @@ impl Proxy {
         self.mocks.push(mock);
     }
 
+    /// Start the proxy server
+    ///
+    /// Will panic if proxy has already been started
     pub fn start(&mut self) {
         start_proxy(self);
     }
 
+    /// Start the server
+    pub fn stop(&mut self) {
+        todo!();
+    }
+
     /// Address and port of the local server.
     /// Can be used with `std::net::TcpStream`.
-    ///
-    /// The server will be started if necessary.
     pub fn address(&self) -> SocketAddr {
         self.listening_addr.expect("server should be listening")
     }
 
     /// A local `http://…` URL of the server.
-    ///
-    /// The server will be started if necessary.
     pub fn url(&self) -> String {
         format!("http://{}", self.address())
     }
 
+    /// Returns the root CA certificate of the server
     pub fn get_certificate(&self) -> Vec<u8> {
-        self.cert.to_pem().unwrap().clone()
+        self.cert.to_pem().unwrap()
     }
 }
 
 #[derive(Debug, Clone)]
-pub struct Request {
+struct Request {
     error: Option<String>,
     path: Option<String>,
     method: Option<String>,
@@ -84,9 +96,10 @@ pub struct Request {
 
 impl std::fmt::Display for Request {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> Result<(), std::fmt::Error> {
-        f.write_str("Request {")?;
-        f.write_char('}')?;
-        Ok(())
+        f.debug_struct("Request")
+            .field("method", &self.method)
+            .field("path", &self.path)
+            .finish_non_exhaustive()
     }
 }
 impl Request {
@@ -97,7 +110,7 @@ impl Request {
         self.error.as_ref()
     }
 
-    fn from(mut stream: Box<&mut dyn Read>) -> Self {
+    fn from(stream: &mut dyn Read) -> Self {
         let mut request = Self {
             error: None,
             path: None,
@@ -141,7 +154,7 @@ impl Request {
                     request.method = req.method.map(|s| s.to_string());
                     request.path = req
                         .path
-                        .map(|s| s.to_string().split(":").next().unwrap().to_owned());
+                        .map(|s| s.to_string().split(':').next().unwrap().to_owned());
                     if let Some(a @ 0..=1) = req.version {
                         request.version = (1, a);
                     }
@@ -166,7 +179,7 @@ fn create_identity(cn: &str, pair: Pair) -> native_tls::Identity {
     native_tls::Identity::from_pkcs12(&encrypted, &password).expect("Unable to build identity")
 }
 
-fn start_proxy<'a>(proxy: &mut Proxy) {
+fn start_proxy(proxy: &mut Proxy) {
     if proxy.started {
         panic!("Tried to start an already started proxy");
     }
@@ -203,7 +216,7 @@ fn start_proxy<'a>(proxy: &mut Proxy) {
         for stream in listener.incoming() {
             info!("Got stream: {:?}", stream);
             if let Ok(mut stream) = stream {
-                let request = Request::from(Box::new(&mut stream));
+                let request = Request::from(&mut stream);
                 info!("Request received: {}", request);
                 if request.is_ok() {
                     handle_request(Pair(cert.as_ref(), pkey.as_ref()), &mocks, request, stream)
@@ -244,7 +257,7 @@ fn open_tunnel<'a>(
     let identity = create_identity(&request.path.unwrap(), identity);
 
     info!("Wrapping with tls");
-    let tstream = native_tls::TlsAcceptor::builder(identity.clone())
+    let tstream = native_tls::TlsAcceptor::builder(identity)
         .build()
         .expect("Unable to build acceptor")
         .accept(stream)
@@ -256,13 +269,13 @@ fn open_tunnel<'a>(
 
 fn handle_request(
     identity: Pair,
-    mocks: &Vec<Mock>,
+    mocks: &[Mock],
     request: Request,
     mut stream: TcpStream,
 ) -> Result<(), Box<dyn std::error::Error>> {
     let mut tstream = open_tunnel(identity, request, &mut stream)?;
 
-    let req = Request::from(Box::new(&mut tstream));
+    let req = Request::from(&mut tstream);
 
     for m in mocks {
         if m.matches(&req) {
@@ -286,9 +299,9 @@ fn write_response(
     for (header, value) in &response.headers {
         tstream.write_fmt(format_args!("{}: {}\r\n", header, value))?;
     }
-    tstream.write(b"\r\n")?;
+    tstream.write_all(b"\r\n")?;
     tstream.write_all(&response.body)?;
-    tstream.write(b"\r\n")?;
+    tstream.write_all(b"\r\n")?;
 
     Ok(())
 }
